@@ -1,6 +1,6 @@
 /* aguix.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2025 Ralf Hoffmann.
+ * Copyright (C) 2001-2026 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -341,7 +341,8 @@ public:
         BOTH_BUFFERS
     };
 
-    void copyToClipboard( const std::string &s, enum target_clipboard_buffer target );
+    int copyToClipboard( const std::string &s, enum target_clipboard_buffer target,
+                         const std::string &mimetype = "" );
     void copyPrimaryToClipboard();
     void copyFileListToClipboard( const std::vector< std::string > &l,
                                   bool cut_operation );
@@ -400,13 +401,15 @@ public:
 
     void startXDND( Widget *initiator,
                     Window w,
-                    unsigned int mouse_button );
+                    unsigned int mouse_button,
+                    unsigned int cancel_mouse_button );
 
     void widgetDestroyed( Widget *w );
 
     int getCurrentUserColorInstance() const;
 
-    bool isModifierPressed( unsigned int mask );
+    bool isModifierPressed( unsigned int mask ) const;
+    bool lastClickModifier( unsigned int mask ) const;
 
     void setStandardIconData( const unsigned long *data,
                               size_t data_length );
@@ -450,7 +453,10 @@ private:
       XA_NET_WM_ICON,
       XA_CARDINAL_ATOM,
       XA_CLIPBOARD,
-      XA_SPECIAL_COPIED_FILES;
+      XA_SPECIAL_COPIED_FILES,
+      XA_INCR;
+
+  std::map< std::string, Atom > m_extra_atoms;
 
   int m_argc;
   char **m_argv;
@@ -547,6 +553,7 @@ private:
     void handleXDNDStatus( XEvent *e );
     void handleXDNDFinished( XEvent *e );
     void cleanupDND();
+    void abortDND();
 
     void initXDNDTypes();
 
@@ -567,6 +574,16 @@ private:
     std::list< RefCount< BackgroundMessageHandler > > m_bg_handler_cleanup_list;
     std::list< AWindow *> m_bg_handler_erase_handler;
 
+    struct copy_requestor_state {
+        Window requestor_window = None;
+        Atom property = None;
+        Atom target_atom = None;
+        size_t buffer_pos = 0;
+        bool in_progress = false;
+        std::string buffer_string;
+        time_t last_update = 0;
+    };
+
     class copy_and_paste_state {
     public:
         void reset() {
@@ -578,11 +595,14 @@ private:
             cutstart = NULL;
             paste_elem = NULL;
             paste_agmessage_data = NULL;
+            target = None;
+            requestors.clear();
         }
 
         std::string buffer_string;
         std::vector< std::string > file_list;
         bool file_list_cut = false;
+        Atom target = None;
 
         Atom last_selection_request_atom = None;
         Window last_selection_request_window = None;
@@ -590,6 +610,13 @@ private:
         GUIElement *cutstart = NULL;
         GUIElement *paste_elem = NULL;
         void *paste_agmessage_data = NULL;
+
+        std::list< struct copy_requestor_state > requestors;
+        copy_requestor_state not_found;
+
+        copy_requestor_state &find_requestor( Window requestor );
+        void remove_requestor( Window requestor );
+        void cleanup_old_requestor( time_t now );
     };
 
     copy_and_paste_state m_primary_buffer_state;
@@ -655,6 +682,8 @@ private:
                              drag_selection_pending( false ),
                              drop_issued( false ),
                              mouse_button( Button1 ),
+                             cancel_mouse_button( Button3 ),
+                             skip_release_count( 0 ),
                              position_silent( false ),
                              position_silent_x( 0 ),
                              position_silent_y( 0 ),
@@ -690,6 +719,8 @@ private:
         bool drop_issued;
 
         unsigned int mouse_button;
+        unsigned int cancel_mouse_button;
+        int skip_release_count;
 
         bool position_silent;
         int position_silent_x;
@@ -708,6 +739,7 @@ private:
     int m_user_color_instance;
 
     unsigned int m_modifier_pressed = 0;
+    unsigned int m_last_click_modifier = 0;
 
     const unsigned long *m_net_wm_icon_data = NULL;
     size_t m_net_wm_icon_data_length = 0;
@@ -715,6 +747,8 @@ private:
     bool msg_handler_active = false;
 
     std::list< std::function< void(void) > > m_post_msg_handler_cbs;
+
+    long m_max_chunk_size = 0;
 };
 
 #endif
