@@ -1,6 +1,6 @@
 /* request.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2025 Ralf Hoffmann.
+ * Copyright (C) 2001-2026 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -63,10 +63,10 @@ public:
   int string_request( const char *title, const char *lines, const char *default_str, const char *buttons,char **return_str, request_flags_t flags );
   int string_request( const char *title, const char *lines, const char *default_str, const char *buttons,char **return_str, const AWindow *twin, request_flags_t flags );
   int string_request( const char *title, const char *lines, const char *default_str, const char *buttons,char **return_str, const AWindow *twin, request_flags_t flags,
-                      struct request_options &options );
+                      const struct request_options &options );
   int string_request( const char *title, const char *lines, const char *default_str, const char *buttons,
                       char **return_str, const AWindow *twin, request_flags_t flags,
-                      struct request_options &options,
+                      const struct request_options &options,
                       const std::string &help_text,
                       const std::function< std::string ( const std::string &current_input_before_cursor,
                                                          const std::string &current_input_after_cursor ) > help_callback );
