@@ -1,6 +1,6 @@
 /* copyop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2025 Ralf Hoffmann.
+ * Copyright (C) 2001-2026 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -47,6 +47,7 @@ CopyOp::CopyOp() : FunctionProto()
   vdir_preserve_dir_structure = false;
   adjust_relative_symlinks = COPYOP_ADJUST_SYMLINK_NEVER;
   ensure_file_permissions = LEAVE_PERMISSIONS_UNMODIFIED;
+  m_also_select_extension = false;
   hasConfigure = true;
     m_category = FunctionProto::CAT_FILEOPS;
     setRequestParameters( false );
@@ -70,6 +71,7 @@ CopyOp::duplicate() const
   ta->vdir_preserve_dir_structure = vdir_preserve_dir_structure;
   ta->adjust_relative_symlinks = adjust_relative_symlinks;
   ta->ensure_file_permissions = ensure_file_permissions;
+  ta->m_also_select_extension = m_also_select_extension;
   ta->setRequestParameters( requestParameters() );
   return ta;
 }
@@ -171,6 +173,10 @@ CopyOp::save(Datei *fh)
           break;
   }
 
+  if ( m_also_select_extension == true ) {
+      fh->configPutPairBool( "alsoselectextension", m_also_select_extension );
+  }
+  
   return true;
 }
 
@@ -216,6 +222,7 @@ CopyOp::normalmodecopy( ActionMessage *am )
     tvdir_preserve_dir_structure = vdir_preserve_dir_structure;
     tadjust_relative_symlinks = adjust_relative_symlinks;
     tensure_file_permissions = ensure_file_permissions;
+    talso_select_extension = m_also_select_extension;
 
     if ( toverwrite != copyorder::COPY_OVERWRITE_NORMAL ) {
         std::string textstr;
@@ -408,6 +415,7 @@ CopyOp::normalmodecopy( ActionMessage *am )
         co->follow_symlinks=tfollow_symlinks;
         co->preserve_attr=tpreserve_attr;
         co->vdir_preserve_dir_structure = tvdir_preserve_dir_structure;
+        co->also_select_extension = talso_select_extension;
 
         switch ( tadjust_relative_symlinks ) {
             case COPYOP_ADJUST_SYMLINK_OUTSIDE:
@@ -426,6 +434,7 @@ CopyOp::normalmodecopy( ActionMessage *am )
         co->do_rename=trename;
         co->overwrite = toverwrite;
         co->destdir = destination_directory.c_str();
+        co->also_select_extension = talso_select_extension;
         // now start copy process
         co->cowin=new CopyOpWin( am->getWorker()->getAGUIX(),
                                        co->move );
@@ -479,8 +488,16 @@ CopyOp::doconfigure(int mode)
   mcb = (ChooseButton*)ac1->add( new ChooseButton( aguix, 0, 0, 20, 20, ( move == true ) ? 1 : 0,
 						   catalog.getLocale( 299 ), LABEL_RIGHT, 0 ), 0, 2, cincwnr );
 
-  rcb = (ChooseButton*)ac1->add( new ChooseButton( aguix, 0, 0, 20, 20, ( do_rename == true ) ? 1 : 0,
-						   catalog.getLocale( 300 ), LABEL_RIGHT, 0 ), 0, 3, cincwnr );
+  AContainer *ac0_3 = ac1->add( new AContainer( win, 2, 1 ), 0, 3 );
+  ac0_3->setMinSpace( 5 );
+  ac0_3->setMaxSpace( 5 );
+  ac0_3->setBorderWidth( 0 );
+
+  rcb = ac0_3->addWidget( new ChooseButton( aguix, 0, 0, 20, 20, ( do_rename == true ) ? 1 : 0,
+                                            catalog.getLocale( 300 ), LABEL_RIGHT, 0 ), 0, 0, cincwnr );
+  auto also_select_extension_cb = ac0_3->addWidget( new ChooseButton( aguix, 0, 0, 20, 20, m_also_select_extension,
+                                                                      catalog.getLocale( 1564 ), LABEL_RIGHT, 0 ), 1, 0, cincwnr );
+
 
   sdcb = (ChooseButton*)ac1->add( new ChooseButton( aguix, 0, 0, 20, 20, ( same_dir == true ) ? 1 : 0,
 						    catalog.getLocale( 301 ), LABEL_RIGHT, 0 ), 0, 4, cincwnr );
@@ -763,6 +780,8 @@ CopyOp::doconfigure(int mode)
               toverwrite = copyorder::COPY_OVERWRITE_NORMAL;
               break;
       }
+
+      talso_select_extension = also_select_extension_cb->getState();
     } else {
       // store in normal variables
       follow_symlinks = fscb->getState();
@@ -822,6 +841,8 @@ CopyOp::doconfigure(int mode)
               overwrite = copyorder::COPY_OVERWRITE_NORMAL;
               break;
       }
+
+      m_also_select_extension = also_select_extension_cb->getState();
     }
   }
   
@@ -938,3 +959,8 @@ void CopyOp::setInteractiveRun()
 {
     setRequestParameters( true );
 }
+
+void CopyOp::setAlsoSelectExtension(bool nv)
+{
+    m_also_select_extension = nv;
+}
