@@ -1,6 +1,6 @@
 /* copyopwin.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2024 Ralf Hoffmann.
+ * Copyright (C) 2001-2026 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -829,12 +829,13 @@ int CopyOpWin::string_request_int( const char *title,
                                    const char *default_str,
                                    const char *buttons,
                                    char **return_str,
-                                   Requester::request_flags_t flags )
+                                   Requester::request_flags_t flags,
+                                   const Requester::request_options &options )
 {
   if ( window != NULL ) {
-    return window->string_request( title, lines, default_str, buttons, return_str, flags );
+      return window->string_request( title, lines, default_str, buttons, return_str, flags, options );
   } else {
-    return Worker::getRequester()->string_request( title, lines, default_str, buttons, return_str, flags );
+      return Worker::getRequester()->string_request( title, lines, default_str, buttons, return_str, NULL, flags, options );
   }
 }
 
@@ -937,9 +938,10 @@ int CopyOpWin::string_request( const std::string &title,
                                const std::string &default_str,
                                const std::string &buttons,
                                std::string &return_str,
-                               Requester::request_flags_t flags )
+                               Requester::request_flags_t flags,
+                               const Requester::request_options &options )
 {
-    BGCopyRequestMessage msg( title, lines, default_str, buttons, flags );
+    BGCopyRequestMessage msg( title, lines, default_str, buttons, flags, options );
 
     if ( std::this_thread::get_id() == m_master_thread_id ) {
         process_request( msg );
@@ -975,7 +977,8 @@ void CopyOpWin::process_request( BGCopyRequestMessage &msg )
                                       msg.getDefaultString().c_str(),
                                       msg.getButtons().c_str(),
                                       &result_string,
-                                      msg.getFlags() );
+                                      msg.getFlags(),
+                                      msg.getOptions() );
 
         if ( result_string ) {
             msg.setResultString( result_string );
