@@ -1,6 +1,6 @@
 /* listermode.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2023 Ralf Hoffmann.
+ * Copyright (C) 2001-2026 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -87,10 +87,14 @@ public:
     typedef enum { LM_GETFILES_ONLYACTIVE,
                    LM_GETFILES_ONLYSELECT,
                    LM_GETFILES_SELORACT} lm_getfiles_t;
+    typedef enum {
+        LM_GETFILES_OPTION_CUSTOM_ATTRIBUTES = 1 << 0,
+    } lm_getfiles_options_t;
 
     virtual int getSelFiles( std::list< NM_specialsourceExt > &return_list,
                              lm_getfiles_t get_mode,
-                             bool unselect = false );
+                             bool unselect = false,
+                             int options = 0 );
 
     virtual std::string getCurrentDirectory();
 
