@@ -35,6 +35,8 @@
 #include "aguix/stringgadget.h"
 #include "aguix/choosebutton.h"
 #include "aguix/button.h"
+#include "aguix/textview.h"
+#include "aguix/awidth.h"
 #include "calendar.hh"
 #include "worker_locale.h"
 #include "wconfig.h"
@@ -46,6 +48,7 @@
 #include "bookmarkdbentry.hh"
 #include "pers_string_list.hh"
 #include "workerinitialsettings.hh"
+#include "textstoragefile.h"
 
 static const int searchopbg_timeout = 10;
 
@@ -66,6 +69,8 @@ std::shared_ptr< PersistentStringList > SearchOpBG::m_match_expression_history;
 
 #define DEFAULT_EDIT_COMMAND "{scripts}/xeditor -line {l} {f}"
 
+#define PREVIEW_DEFAULT_SIZE ( 16 * 1024 )
+
 void SearchOpBG::clear_stored_results()
 {
     for ( std::list<ResultStore*>::iterator it1 = _stored_results.begin();
@@ -199,6 +204,8 @@ bool SearchOpBG::consumeResult( SearchThread::SearchResult &result, FieldListVie
     if ( row == 0 ) {
         lv.setActiveRow( row );
         storeActiveRow( row );
+
+        previewResult();
     }
 
     return new_result;
@@ -1210,7 +1217,12 @@ void SearchOpBG::createWindow()
     ac1_res->setBorderWidth( 0 );
     ac1->setHWeight( 2, 0, 1 );
 
-    _resultlv = (FieldListView*)ac1_res->add( new FieldListView( aguix, 0, 0, 100, 100, 0 ), 0, 0, AContainer::CO_MIN );
+    AContainer *ac1_res_s1 = ac1_res->add( new AContainer( m_win, 2, 1 ), 0, 0 );
+    ac1_res_s1->setMinSpace( 0 );
+    ac1_res_s1->setMaxSpace( 0 );
+    ac1_res_s1->setBorderWidth( 0 );
+
+    _resultlv = (FieldListView*)ac1_res_s1->add( new FieldListView( aguix, 0, 0, 100, 100, 0 ), 0, 0, AContainer::CO_MIN );
     _resultlv->setNrOfFields( 3 );
     _resultlv->setFieldWidth( 1, 5 );
     _resultlv->setFieldText( 0, catalog.getLocale( 718 ) );
@@ -1224,6 +1236,25 @@ void SearchOpBG::createWindow()
     _resultlv->setDisplayFocus( true );
     _resultlv->setMBG( aguix->getFaces().getColor( "dirview-bg" ) );
 
+    AContainer *ac1_res_s1_2 = ac1_res_s1->add( new AContainer( m_win, 1, 2 ), 1, 0 );
+    ac1_res_s1_2->setMinSpace( 5 );
+    ac1_res_s1_2->setMaxSpace( 5 );
+    ac1_res_s1_2->setBorderWidth( 5 );
+
+    m_result_preview_cb = ac1_res_s1_2->addWidget( new ChooseButton( aguix, 0, 0, false,
+                                                                     catalog.getLocale( 1611 ),
+                                                                     LABEL_LEFT, 0 ), 0, 0, AContainer::CO_INCWNR );
+
+    m_result_preview_tv = ac1_res_s1_2->addWidget( new TextView( aguix, 0, 0, 100, 100, "", nullptr ), 0, 1, AContainer::CO_MIN );
+    m_result_preview_tv->setFont( wconfig->getFont( 4 ).c_str() );
+    m_result_preview_tv->create();
+    m_result_preview_tv->show();
+    m_result_preview_tv->setAcceptFocus( true );
+    m_result_preview_tv->setDisplayFocus( true );
+
+    m_lencalc.reset( new AFontWidth( aguix, aguix->getFont( wconfig->getFont( 4 ).c_str() ) ) );
+    m_ts_file = std::make_shared< TextStorageFile >( "", m_lencalc, PREVIEW_DEFAULT_SIZE, TextStorageString::CONVERT_NON_PRINTABLE_SIMPLE );
+
     AContainer *ac1_4 = ac1_res->add( new AContainer( m_win, 2, 1 ), 0, 1 );
     ac1_4->setMinSpace( 0 );
     ac1_4->setMaxSpace( 0 );
@@ -1484,6 +1515,8 @@ void SearchOpBG::handleAGUIXMessage( AGMessage &msg )
                 int row = _resultlv->getActiveRow();
                 if ( _resultlv->isValidRow( row ) == true ) {
                     storeActiveRow( row );
+
+                    previewResult();
                 }
             } else if ( msg.fieldlv.lv == m_history_lv ) {
                 int row = m_history_lv->getActiveRow();
@@ -1572,9 +1605,25 @@ void SearchOpBG::handleAGUIXMessage( AGMessage &msg )
                     advanceHistoryItem( 1 );
                 } else if ( msg.key.key == XK_Up ) {
                     advanceHistoryItem( -1 );
+                } else if ( msg.key.key == XK_p &&
+                            KEYSTATEMASK( msg.key.keystate ) == ControlMask ) {
+                    m_result_preview_cb->setState( ! m_result_preview_cb->getState() );
+                    previewResult();
                 }
             }
             break;
+    case AG_TEXTVIEW_END_REACHED:
+        if ( msg.textview.textview == m_result_preview_tv &&
+             m_ts_file->incompleteFile() ) {
+            m_ts_file->readMore();
+            m_result_preview_tv->textStorageChanged();
+        }
+        break;
+    case AG_CHOOSECLICKED:
+        if ( msg.choose.button == m_result_preview_cb ) {
+            previewResult();
+        }
+        break;
     }
 
     if ( endmode == 2 || endmode == 3 ) {
@@ -1911,3 +1960,32 @@ void SearchOpBG::removeHistoryItem( bool all )
         updateHistory( m_match_expression_sg, false );
     }
 }
+
+void SearchOpBG::previewResult()
+{
+    if ( ! m_result_preview_cb->getState() ) {
+        m_ts_file->resetContent();
+        m_result_preview_tv->unsetHighlightLine();
+        m_result_preview_tv->replaceTextStorage( m_ts_file );
+        return;
+    }
+
+    int row = _resultlv->getActiveRow();
+    if ( _resultlv->isValidRow( row ) ) {
+        SearchThread::SearchResult r = getResultForRow( row );
+        if ( r.getFullname().length() > 0 ) {
+            m_ts_file->setFileName( r.getFullname(), PREVIEW_DEFAULT_SIZE );
+            m_result_preview_tv->replaceTextStorage( m_ts_file );
+            m_result_preview_tv->textStorageChanged();
+
+            if ( r.getLineNr() >= 0 ) {
+                while ( ! m_result_preview_tv->jumpToLine( r.getLineNr(), true ) ) {
+                    if ( ! m_ts_file->incompleteFile() ) break;
+
+                    m_ts_file->readMore();
+                    m_result_preview_tv->textStorageChanged();
+                }
+            }
+        }
+    }
+}
