@@ -1,6 +1,6 @@
 /* showimagemode.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2015 Ralf Hoffmann.
+ * Copyright (C) 2001-2025 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -32,6 +32,7 @@
 #include "aguix/stringgadget.h"
 #include "aguix/button.h"
 #include "aguix/fieldlistview.h"
+#include "aguix/choosebutton.h"
 #include "aguix/awindow.h"
 #include "datei.h"
 #include "simplelist.hh"
@@ -68,6 +69,18 @@ void ShowImageMode::messageHandler(AGMessage *msg)
       parentlister->getGeometry(&tx,&ty,&tw,&th);
       reconf(tx,ty,tw,th);
       break;
+  case AG_EXPOSE:
+      if ( imgwin ) {
+          if ( msg->expose.window == imgwin->getWindow() ) {
+#ifdef HAVE_IMLIB2
+              if ( m_use_internal && m_buffer ) {
+                  imlib_context_set_image(m_buffer);
+                  imlib_render_image_on_drawable( 0, 0 );
+              }
+#endif
+          }
+      }
+      break;
   }
   if(ma==true) parentlister->makeActive();
 }
@@ -139,6 +152,8 @@ void ShowImageMode::off()
   delete imgwin;
   imgwin=NULL;
   clearLastActiveFE();
+
+  destroy_internal_viewer();
 }
 
 void ShowImageMode::reconf(int tx,int ty,int tw,int th)
@@ -187,19 +202,7 @@ int ShowImageMode::configure()
   AGMessage *msg;
   int endmode=-1;
   char *tstr;
-  char **liness;
-  int lines;
   Requester *req;
-  int i;
-  const int cincw = AContainer::ACONT_MINH +
-                    AContainer::ACONT_MINW +
-                    AContainer::ACONT_MAXH;
-  const int cincwnr = cincw +
-                      AContainer::ACONT_NORESIZE;
-  const int cfix = AContainer::ACONT_MINH +
-                   AContainer::ACONT_MINW +
-                   AContainer::ACONT_MAXH +
-                   AContainer::ACONT_MAXW;
   
   tstr = (char*)_allocsafe( strlen( catalog.getLocale( 293 ) ) + strlen( getLocaleName() ) + 1 );
   sprintf( tstr, catalog.getLocale( 293 ), getLocaleName() );
@@ -207,21 +210,29 @@ int ShowImageMode::configure()
   win->create();
   _freesafe(tstr);
 
-  lines=createLines(catalog.getLocale(521),&liness);
-
-  AContainer *ac1 = win->setContainer( new AContainer( win, 1, lines + 3 ), true );
+  AContainer *ac1 = win->setContainer( new AContainer( win, 1, 5 ), true );
   ac1->setMinSpace( 5 );
   ac1->setMaxSpace( 5 );
 
-  for(i=0;i<lines;i++) {
-    ac1->add( new Text( aguix, 0, 0, liness[i] ), 0, i, cincwnr );
-  }
+  ChooseButton *internal_cb = nullptr;
+#ifdef HAVE_IMLIB2
+  internal_cb = ac1->addWidget( new ChooseButton( aguix, 0, 0,
+                                                  m_use_internal,
+                                                  catalog.getLocale( 1608 ),
+                                                  LABEL_RIGHT, 0 ), 0, 0, AContainer::CO_INCWNR );
+#else
+  ac1->setMaxHeight( 0, 0, 0 );
+  m_use_internal = false;
+#endif
 
-  sg = (StringGadget*)ac1->add( new StringGadget( aguix, 0, 0, 100, show_program, 0 ), 0, lines, cincw );
+  AContainer *info_text_ac = nullptr;
+  win->addMultiLineText( catalog.getLocale( 521 ), *ac1, 0, 1, &info_text_ac, NULL );
+
+  sg = ac1->addWidget( new StringGadget( aguix, 0, 0, 100, show_program, 0 ), 0, 2, AContainer::CO_INCW );
   
-  db = (Button*)ac1->add( new Button( aguix, 0, 0, 100, catalog.getLocale( 110 ), 0 ), 0, lines + 1, cincw );
+  db = ac1->addWidget( new Button( aguix, 0, 0, 100, catalog.getLocale( 110 ), 0 ), 0, 3, AContainer::CO_INCW );
 
-  AContainer *ac1_2 = ac1->add( new AContainer( win, 2, 1 ), 0, lines + 2 );
+  AContainer *ac1_2 = ac1->add( new AContainer( win, 2, 1 ), 0, 4 );
   ac1_2->setMinSpace( 5 );
   ac1_2->setMaxSpace( -1 );
   ac1_2->setBorderWidth( 0 );
@@ -229,16 +240,23 @@ int ShowImageMode::configure()
                                                 0,
                                                 0,
                                                 catalog.getLocale( 11 ),
-                                                0 ), 0, 0, cfix );
+                                                0 ), 0, 0, AContainer::CO_FIX );
   Button *cb = (Button*)ac1_2->add( new Button( aguix,
 						0,
 						0,
 						catalog.getLocale( 8 ),
-						0 ), 1, 0, cfix );
-  
+						0 ), 1, 0, AContainer::CO_FIX );
+
   win->setDoTabCycling( true );
   win->contMaximize( true );
   win->show();
+
+  if ( m_use_internal ) {
+      info_text_ac->hide();
+      sg->hide();
+      db->hide();
+  }
+
   req=new Requester(aguix);
   for(;endmode==-1;) {
     msg=aguix->WaitMessage(win);
@@ -259,6 +277,18 @@ int ShowImageMode::configure()
             }
           }
           break;
+      case AG_CHOOSECLICKED:
+          if ( msg->choose.button == internal_cb ) {
+              if ( msg->choose.state ) {
+                  info_text_ac->hide();
+                  sg->hide();
+                  db->hide();
+              } else {
+                  info_text_ac->show();
+                  sg->show();
+                  db->show();
+              }
+          }
       }
       aguix->ReplyMessage(msg);
     }
@@ -269,13 +299,13 @@ int ShowImageMode::configure()
     // ok
     _freesafe(show_program);
     show_program=dupstring(sg->getText());
+    if ( internal_cb ) {
+        m_use_internal = internal_cb->getState();
+    }
   }
   
   delete win;
 
-  for(i=0;i<lines;i++) _freesafe(liness[i]);
-  _freesafe(liness);
-
   return endmode;
 }
 
@@ -296,21 +326,42 @@ const char* ShowImageMode::getStaticLocaleName()
 
 int ShowImageMode::load()
 {
-  if ( worker_token == VIEWSTR_WCP ) {
-    readtoken();
+    for (;;) {
+        if ( worker_token == VIEWSTR_WCP ) {
+            readtoken();
 
-    if ( worker_token != '=' ) return 1;
-    readtoken();
+            if ( worker_token != '=' ) return 1;
+            readtoken();
 
-    if ( worker_token != STRING_WCP ) return 1;
-    if ( show_program != NULL ) _freesafe( show_program );
-    show_program = dupstring( yylval.strptr );
-    readtoken();
+            if ( worker_token != STRING_WCP ) return 1;
+            if ( show_program != NULL ) _freesafe( show_program );
+            show_program = dupstring( yylval.strptr );
+            readtoken();
 
-    if ( worker_token != ';' ) return 1;
-    readtoken();
-  } else return 1; //parse error
-  return 0;
+            if ( worker_token != ';' ) return 1;
+            readtoken();
+        } else if ( worker_token == INTERNALVIEWER_WCP ) {
+            readtoken();
+
+            if ( worker_token != '=' ) return 1;
+            readtoken();
+
+            if ( worker_token == YES_WCP ) {
+                m_use_internal = true;
+            } else if ( worker_token == NO_WCP ) {
+                m_use_internal = false;
+            } else {
+                return 1;
+            }
+            readtoken();
+
+            if ( worker_token != ';' ) return 1;
+            readtoken();
+        } else {
+            break;
+        }
+    }
+    return 0;
 }
 
 bool ShowImageMode::save(Datei *fh)
@@ -318,6 +369,11 @@ bool ShowImageMode::save(Datei *fh)
   if ( fh == NULL ) return false;
   
   fh->configPutPairString( "viewstr", show_program );
+#ifdef HAVE_IMLIB2
+  if ( ! m_use_internal ) {
+      fh->configPutPairBool( "internalviewer", m_use_internal );
+  }
+#endif
 
   return false;
 }
@@ -333,78 +389,140 @@ void ShowImageMode::relayout()
 
 void ShowImageMode::update(bool force)
 {
-  ListerMode *lm1=NULL;
-  Lister *ol=NULL;
-  List *colors;
-  WC_Color *bgcol;
-  const FileEntry *fe;
-  char *buf,*colbuf, *buf2;
-  std::string str1;
+    ListerMode *lm1=NULL;
+    Lister *ol=NULL;
+    List *colors;
+    WC_Color *bgcol;
+    const FileEntry *fe;
+    char *buf,*colbuf, *buf2;
+    std::string str1;
+
+    ol=parentlister->getWorker()->getOtherLister(parentlister);
+    if(ol!=NULL) {
+        lm1=ol->getActiveMode();
+    }
   
-  ol=parentlister->getWorker()->getOtherLister(parentlister);
-  if(ol!=NULL) {
-    lm1=ol->getActiveMode();
-  }
-  
-  if(force==true) clearLastActiveFE();
-  
-  if(lm1!=NULL) {
+    if(force==true) clearLastActiveFE();
+
+    if ( ! lm1 ) return;
+
     std::list< NM_specialsourceExt > files;
     
     lm1->getSelFiles( files, ListerMode::LM_GETFILES_ONLYACTIVE );
-    if ( ! files.empty() ) {
-      fe = files.begin()->entry();
-      if ( fe->equals( lastactivefe ) == false ) {
-	clearLastActiveFE();
-	if ( fe != NULL ) {
-	  lastactivefe = new FileEntry( *fe );
-	}
-        if(lastactivefe!=NULL) {
-          // first get the background color in the X format for xli
-          colors=(List*)wconfig->getColors();
-          bgcol=(WC_Color*)colors->getElementAt(0);
-          colbuf = (char*)_allocsafe( strlen( "rgb://" ) + ( 3 * A_BYTESFORNUMBER( int ) ) + 1 );
-          sprintf( colbuf, "rgb:%x/%x/%x", bgcol->getRed(), bgcol->getGreen(), bgcol->getBlue() );
+    if ( files.empty() ) {
+        clearLastActiveFE();
+        blank();
+        return;
+    }
+
+    fe = files.begin()->entry();
+    if ( fe->equals( lastactivefe ) ) {
+        return;
+    }
+
+    clearLastActiveFE();
+
+    if ( fe != NULL ) {
+        lastactivefe = new FileEntry( *fe );
+    }
+
+    init_internal_viewer();
+
+#ifdef HAVE_IMLIB2
+    if ( ! m_use_internal ) {
+#endif
+        // first get the background color in the X format for xli
+        colors=(List*)wconfig->getColors();
+        bgcol=(WC_Color*)colors->getElementAt(0);
+        colbuf = (char*)_allocsafe( strlen( "rgb://" ) + ( 3 * A_BYTESFORNUMBER( int ) ) + 1 );
+        sprintf( colbuf, "rgb:%x/%x/%x", bgcol->getRed(), bgcol->getGreen(), bgcol->getBlue() );
         
-          if ( show_program[0] != '/' ) {
+        if ( show_program[0] != '/' ) {
             // no absolute path so check if we find
             // the file in the script directory
             str1 = Worker::getDataDir();
             str1 += "/scripts/";
             str1 += show_program;
             if ( Datei::fileExistsExt( str1.c_str() ) != Datei::D_FE_FILE ) {
-              // not found so let the shell find it
-              str1 = show_program;
+                // not found so let the shell find it
+                str1 = show_program;
             }
-          } else {
+        } else {
             str1 = show_program;
-          }
-
-          buf = (char*)_allocsafe( strlen( "%s %d %d %d %d %x " ) +
-                                   strlen( str1.c_str() ) +
-                                   4 * A_BYTESFORNUMBER( int ) + 
-                                   A_BYTESFORNUMBER( Window ) + 1 );
-          sprintf( buf, "%s %d %d %d %d %x ", str1.c_str(), 0, 0, w, h, (unsigned int)imgwin->getWindow() );
-          buf2 = AGUIX_catTrustedAndUnTrusted( buf, lastactivefe->fullname );
-          _freesafe( buf );
-          buf = catstring( buf2, " " );
-          _freesafe( buf2 );
-          buf2 = catstring( buf, colbuf );
-          _freesafe( buf );
-          buf = buf2;
-          blank();
-          int tres __attribute__((unused)) = system( buf );
-          blanked=false;
-          
-          _freesafe(buf);
-          _freesafe(colbuf);
         }
-      }
+
+        buf = (char*)_allocsafe( strlen( "%s %d %d %d %d %x " ) +
+                                 strlen( str1.c_str() ) +
+                                 4 * A_BYTESFORNUMBER( int ) + 
+                                 A_BYTESFORNUMBER( Window ) + 1 );
+        sprintf( buf, "%s %d %d %d %d %x ", str1.c_str(), 0, 0, w, h, (unsigned int)imgwin->getWindow() );
+        buf2 = AGUIX_catTrustedAndUnTrusted( buf, lastactivefe->fullname );
+        _freesafe( buf );
+        buf = catstring( buf2, " " );
+        _freesafe( buf2 );
+        buf2 = catstring( buf, colbuf );
+        _freesafe( buf );
+        buf = buf2;
+        blank();
+        int tres __attribute__((unused)) = system( buf );
+        blanked=false;
+          
+        _freesafe(buf);
+        _freesafe(colbuf);
+#ifdef HAVE_IMLIB2
     } else {
-      clearLastActiveFE();
-      blank();
+        Imlib_Image image;
+
+        blank();
+
+        imlib_context_set_blend(0);
+        imlib_context_set_color_modifier(NULL);
+        imlib_context_set_drawable( imgwin->getWindow() );
+        image = imlib_load_image( lastactivefe->fullname );
+        if ( image ) {
+            imlib_context_set_image(image);
+            imlib_image_set_changes_on_disk();
+            int iw = imlib_image_get_width();
+            int ih = imlib_image_get_height();
+
+            if ( iw > 0 && ih > 0 ) {
+                double scale_w = (double)w / (double)iw;
+                double scale_h = (double)h / (double)ih;
+                int target_w, target_h;
+
+                if ( scale_w < scale_h ) {
+                    target_w = w;
+                    target_h = floor( ih * scale_w );
+                } else {
+                    target_h = h;
+                    target_w = floor( iw * scale_h );
+                }
+
+                if ( m_buffer ) {
+                    imlib_context_set_image(m_buffer);
+                    imlib_free_image();
+                    m_buffer = nullptr;
+                }
+
+                m_buffer = imlib_create_image( target_w, target_h );
+                imlib_context_set_blend(0);
+
+                imlib_context_set_image(m_buffer);
+                imlib_blend_image_onto_image(image, 0,
+                                             0, 0, iw, ih,
+                                             0, 0, target_w, target_h);
+                imlib_context_set_image(image);
+                imlib_free_image();
+                imlib_context_set_blend(0);
+                imlib_context_set_image(m_buffer);
+                imlib_render_image_on_drawable(0,0);
+                blanked = false;
+            } else {
+                imlib_free_image();
+            }
+        }
     }
-  }
+#endif
 }
 
 void ShowImageMode::blank()
@@ -531,3 +649,49 @@ void ShowImageMode::clearLastActiveFE() {
     lastactivefe = NULL;
   }
 }
+
+static int worker_imlib_progress( Imlib_Image im,
+                                  char percent,
+                                  int update_x, int update_y,
+                                  int update_w, int update_h)
+{
+    Worker::getAGUIX()->doXMsgs( NULL, true );
+
+    if ( Worker::getAGUIX()->getLastKeyRelease() == XK_Escape ) {
+        return 0;
+    }
+
+    return 1;
+}
+
+void ShowImageMode::init_internal_viewer()
+{
+#ifdef HAVE_IMLIB2
+  if ( m_use_internal && ! m_imlib_initialized ) {
+      imlib_context_set_display( aguix->getDisplay());
+      imlib_context_set_visual( DefaultVisual( aguix->getDisplay(), aguix->getScreen() ) );
+      imlib_context_set_colormap( aguix->getColormap() );
+      imlib_set_cache_size( 2048 * 1024 );
+      imlib_set_color_usage( 128 );
+      imlib_context_set_dither( 1 );
+      imlib_context_set_progress_function( worker_imlib_progress );
+
+      m_imlib_initialized = true;
+  }
+#endif
+}
+
+void ShowImageMode::destroy_internal_viewer()
+{
+#ifdef HAVE_IMLIB2
+    if ( m_imlib_initialized ) {
+        imlib_set_cache_size(0);
+        if ( m_buffer ) {
+            imlib_context_set_image( m_buffer );
+            imlib_free_image();
+            m_buffer = nullptr;
+        }
+        m_imlib_initialized = false;
+    }
+#endif
+}
