@@ -371,6 +371,8 @@ public:
 
     const WConfigKVPPool &getKVPPool() const;
     void setKVPPool( const WConfigKVPPool &kvp );
+
+    void applyKVPPool();
 protected:
   char *lang;
   unsigned int rows,columns;
@@ -532,7 +534,7 @@ protected:
 
     std::string m_disable_bg_check_prefix;
 
-    WConfigKVPPool m_kvp_pool;
+    std::shared_ptr< WConfigKVPPool > m_kvp_pool;
 };
 
 extern WConfig *wconfig;
